<?php
/**
 *
 */
class eventHandler
{
    /**
     *
     * @return
     */
    function add()
    {

        $validator = new eventValidator();
        $tempEvent = new event();
        $continue = true;
        $errorMessage = '';

        if (wp_verify_nonce($_POST['_wpnonce'], 'editEvent'))
        {

            //===================================================
            // Validate the Name
            //===================================================

            if ($validator->validEventName($_POST['eventName']))
            {
                $tempEvent->setEventName($_POST['eventName']);
            }
            else
            {
                $continue = false;
                $errorMessage .= 'The name you have entered is invalid<br />';
            }

            //===================================================
            // Validate the Description
            //===================================================
            if ($validator->validEventDescription($_POST['eventDescription']))
            {
                $tempEvent->setEventDescription($_POST['eventDescription']);
            }
            else
            {
                $continue = false;
                $errorMessage .= 'The description you have entered is invalid<br />';
            }

            //===================================================
            // Validate the Excerpt
            //===================================================
            if ($validator->validEventExcerpt($_POST['eventExcerpt']))
            {
                $tempEvent->setEventExcerpt($_POST['eventExcerpt']);
            }
            else
            {
                $continue = false;
                $errorMessage .= 'The excerpt you have entered is invalid<br />';
            }


            //===================================================
            // Convert the Time
            //===================================================

            //Retrieve the time fields and turn them into one cohesive field
            $eventStartTimeTemp = $_POST['startTimeHour']+$_POST['startTimeAMPMSelect'].':'.$_POST['startTimeMin'];
            $eventEndTimeTemp = $_POST['endTimeHour']+$_POST['endTimeAMPMSelect'].':'.$_POST['endTimeMin'];


            //Turn these fields into timestamps
            $eventStartTimeTemp = eventHandler::processTime($eventStartTimeTemp);
            $eventEndTimeTemp = eventHandler::processTime($eventEndTimeTemp);


            //Change the timestamps to GMT for storage in the DB
            $timeManager = new dateTimeManager();

            $eventStartTimeTemp = $timeManager->convertToSystemTime($eventStartTimeTemp);
            $eventEndTimeTemp = $timeManager->convertToSystemTime($eventEndTimeTemp);


            //echo 'Start time right after conversion: ' . date('g:i a', $eventStartTimeTemp) . '<br />';
            //echo 'End  &nbsp; time right after conversion: ' . date('g:i a', $eventEndTimeTemp). '<br />';
            //===================================================
            //Validate the date
            //===================================================

            if (!$validator->validDate($_POST['eventDate']))
            {
                $continue = false;
                $errorMessage .= 'The date you specified for this event is invalid';
            }

            //===================================================
            //Validate the start and end time
            //===================================================
            if ($validator->validDates($eventStartTimeTemp, $eventEndTimeTemp))
            {
                //Store the times in the object
                $tempEvent->setEventStartTime($eventStartTimeTemp);
                $tempEvent->setEventEndTime($eventEndTimeTemp);
            }
            else
            {
                $continue = false;
                $errorMessage .= 'The times you have selected for this event are invalid';
            }

            //===================================================
            // Process remaining fields
            //===================================================
            //Check if the checkbox for it being an all day event is checked
            if ( isset ($_POST['allDayEvent']))
            {
                $tempEvent->setIsAllDay(true);
            }
            else
            {
                $tempEvent->setIsAllDay(false);
            }

            $tempEvent->setRepeatFrequency($_POST['eventFrequency']);
            $tempEvent->setRepeatTimes($_POST['eventRepeatTimes']);
            $tempEvent->setrepeatEnd($_POST['eventFrequency']);

            //Check if the checkbox for it repeating forever is checked
            if ( isset ($_POST['repeatForever']))
            {
                $tempEvent->setrepeatForever(true);
            }
            else
            {
                $tempEvent->setrepeatForever(false);
            }

            $tempEvent->setLocation($_POST['eventLocation']);

            if ( isset ($_POST['eventCalendar']))
            {
                $tempEvent->setCalendarID($_POST['eventCalendar']);
            }

        }
        else
        {
            $continue = false;
            $errorMessage = 'Transaction validation failed';
        }


        //Only write the data to the DB if the data is actually valid!
        if ($continue)
        {

            //Add the event, get the event ID from the add method
            $eventID = $tempEvent->add();


            //If the user specified a list of categories this event belongs to, then store that

            if ( isset ($_POST['categoryList']))
            {
                $calendarList = array ();

                $i = 0;
                foreach ($_POST['categoryList'] as $calendarID)
                {
                    $eventCalendarMeta = new eventCalendarMeta();
                    $eventCalendarMeta->setCalendarID($calendarID);
                    $eventCalendarMeta->setEventID($eventID);
                    $eventCalendarMeta->add();
                }

                $tempEvent->setCalendars($calendarList);
            }
            $page = new wec_backendListView();
            $page->display();


        }

        //Otherwise, go back to the same page in order to edit the fields
        else
        {
            raiserror($errorMessage);
            wec_FixInvalidEvent($_POST);
        }


    }

    /**
     *
     * @return
     */
    function read($eventID)
    {
        if ($eventID == null)
        {
            throw new Exception('No event ID specified for the read function of the event interface');
        }
        else
        {
            $tempEvent = new event($eventID);
        }

        return $tempEvent;

    }

    function readAll()
    {
        $eventDA = new eventDA();
        return $eventDA->getAllEvents();
    }

    /**
     *
     * @return
     * @param object $eventID[optional]
     */
    function update()
    {

        $validator = new eventValidator();
        $tempEvent = new event($_POST['eventID']);
        $continue = true;
        $errorMessage = '';

        //===================================================
        // Validate the Name
        //===================================================

        if ($validator->validEventName($_POST['eventName']))
        {
            $tempEvent->setEventName($_POST['eventName']);
        }
        else
        {
            $continue = false;
            $errorMessage .= 'The name you have entered is invalid<br />';
        }

        //===================================================
        // Validate the Description
        //===================================================
        if ($validator->validEventDescription($_POST['eventDescription']))
        {
            $tempEvent->setEventDescription($_POST['eventDescription']);
        }
        else
        {
            $continue = false;
            $errorMessage .= 'The description you have entered is invalid<br />';
        }

        //===================================================
        // Validate the Excerpt
        //===================================================
        if ($validator->validEventExcerpt($_POST['eventExcerpt']))
        {
            $tempEvent->setEventExcerpt($_POST['eventExcerpt']);
        }
        else
        {
            $continue = false;
            $errorMessage .= 'The excerpt you have entered is invalid<br />';
        }


        //===================================================
        // Convert the Time
        //===================================================

        //Retrieve the time fields and turn them into one cohesive field
        $eventStartTimeTemp = $_POST['startTimeHour']+$_POST['startTimeAMPMSelect'].':'.$_POST['startTimeMin'];
        $eventEndTimeTemp = $_POST['endTimeHour']+$_POST['endTimeAMPMSelect'].':'.$_POST['endTimeMin'];

        //echo 'Start time from form: ' . $eventStartTimeTemp . ' <br />';
        //echo 'End &nbsp;time from form: ' . $eventEndTimeTemp . '<br />';


        //Turn these fields into timestamps
        $eventStartTimeTemp = eventHandler::processTime($eventStartTimeTemp);
        $eventEndTimeTemp = eventHandler::processTime($eventEndTimeTemp);

        //Change the timestamps to GMT for storage in the DB
        $timeManager = new dateTimeManager();

        //echo 'Start time before conversion: ' . $eventStartTimeTemp . ' <br />';
        //echo 'End &nbsp;time before conversion: ' . $eventEndTimeTemp . '<br />';


        $eventStartTimeTemp = $timeManager->convertToSystemTime($eventStartTimeTemp);
        $eventEndTimeTemp = $timeManager->convertToSystemTime($eventEndTimeTemp);




        //===================================================
        //Validate the date
        //===================================================

        if (!$validator->validDate($_POST['eventDate']))
        {
            $continue = false;
            $errorMessage .= 'The date you specified for this event is invalid';
        }

        //===================================================
        //Validate the start and end time
        //===================================================
        if ($validator->validDates($eventStartTimeTemp, $eventEndTimeTemp))
        {
            //Store the times in the object
            $tempEvent->setEventStartTime($eventStartTimeTemp);
            $tempEvent->setEventEndTime($eventEndTimeTemp);
        }
        else
        {
            $continue = false;
            $errorMessage .= 'The times you have selected for this event are invalid';
        }

        //===================================================
        // Process remaining fields
        //===================================================
        //Check if the checkbox for it being an all day event is checked
        if ( isset ($_POST['allDayEvent']))
        {
            $tempEvent->setIsAllDay(true);
        }
        else
        {
            $tempEvent->setIsAllDay(false);
        }

        //===================================================
        // Handle changing repeat frequency or times
        //===================================================
        $clearRecurrences = false;
        //If our frequency has changed, we need to clear all recurrences and change it, otherwise we'll leave it the same
        if (strcmp($tempEvent->getRepeatFrequency(), $_POST['eventFrequency']) != 0)
        {
            $clearRecurrences = true;
            $tempEvent->setRepeatFrequency($_POST['eventFrequency']);
        }
        //Same deal, if it's changed, redo all recurrences
        if ($_POST['eventRepeatTimes'] == $tempEvent->getRepeatTimes())
        {
            $clearRecurrences = true;
            $tempEvent->setRepeatTimes($_POST['eventRepeatTimes']);
        }

        //If anything significant changed then redo all the recurrences
        if ($clearRecurrences)
        {
            $recurrenceHandler = new recurrenceDA();
            $recurrenceHandler->deleteAllRecurrences($_POST['eventID']);
        }


        $tempEvent->setrepeatEnd($_POST['eventFrequency']);

        //Check if the checkbox for it repeating forever is checked
        if ( isset ($_POST['repeatForever']))
        {
            $tempEvent->setrepeatForever(true);
        }
        else
        {
            $tempEvent->setrepeatForever(false);
        }

        $tempEvent->setLocation($_POST['eventLocation']);

        if ( isset ($_POST['eventCalendar']))
        {
            $tempEvent->setCalendarID($_POST['eventCalendar']);
        }


        //Only write the data to the DB if the data is actually valid!
        if ($continue)
        {

            //Add the event, get the event ID from the add method
            $tempEvent->update();
            adminAlertFade($tempEvent->getEventName().' has been updated');

            //Clear out the set calendars for this event, in order to write the new ones
            $eventCalendarMetaDA = new eventCalendarMetaDA();
            $eventCalendarMetaDA->delete($_POST['eventID']);

            //If the user specified a list of categories this event belongs to, then store that

            if ( isset ($_POST['categoryList']))
            {
                $calendarList = array ();

                $i = 0;
                foreach ($_POST['categoryList'] as $calendarID)
                {
                    $eventCalendarMeta = new eventCalendarMeta();
                    $eventCalendarMeta->setCalendarID($calendarID);
                    $eventCalendarMeta->setEventID($_POST['eventID']);
                    $eventCalendarMeta->add();
                }

                $tempEvent->setCalendars($calendarList);
            }


            //Once we're done, go to the default view
            $listview = new wec_backendListView();
            $listview->display();

        }

        //Otherwise, go back to the same page in order to edit the fields
        else
        {
            raiserror($errorMessage);
            wec_FixInvalidEvent($_POST);
        }


    }

    /**
     *
     * @return
     */
    function delete($eventID = null)
    {

        if ($eventID == null)
        {
            throw new Exception('No event ID specified for the delete function of the event interface');
        }
        else
        {
            $tempEvent = new event($eventID, false);
            $tempEvent->delete();
        }

    }

    /**
     *
     * @return The results of the add() function. This is just a wrapper.
     */
    function create()
    {
        eventInterface::add();
    }

    private function processTime($timeField)
    {

        $timeManager = new dateTimeManager();

        //Turn the event time into an array of values with hours and minutes
        $startTimeArray = explode(':', $timeField);
        $hours = $startTimeArray[0];
        $minutes = $startTimeArray[1];

        //DateTimeStamp is a UNIX timestamp that represents the date of the event
        //in user time, with the comma removed. We implode and then explode it
        //to remove the comma.
        $dateTimeStamp = strtotime(implode('', explode(',', $_POST['eventDate'])));

        $yearNumber = $timeManager->getYear($dateTimeStamp);
        $monthNumber = $timeManager->getMonthNumber($dateTimeStamp);
        $dayNumber = $timeManager->getDayOfMonthNumber($dateTimeStamp);


        $timestamp = mktime($hours, $minutes, 0, $monthNumber, $dayNumber, $yearNumber);

        return $timestamp;

    }

    private function processBoolean($field)
    {

        //Check if the value is zero, if it is, set it to false, otherwise, set it to true
        if ( empty($field))
        {
            $temp = false;
        } else
        {
            $temp = true;
        }

        return $temp;
    }

    function lookupCalendarsForEvent($eventID = null)
    {
        if ( isset ($eventID))
        {
            $eventCalendarMetaDA = new eventCalendarMetaDA();
            return $eventCalendarMetaDA->lookupCalendarsForEvent($eventID);
        }
        else
        {
            throw new Exception('You must enter an eventID');
        }
    }

}

?>
